#!/usr/bin/perl
#
# Daniel Stenberg <daniel@haxx.se>
$date=   "October 3, 2000";
$version="0.21";
#
# Spam is an internet desease that's really hard to take down. I do my best to
# complain on all spam mails I receive. 1997 I wrote a script that does the
# complaining for me. This is that script, rewritten in perl with improved
# functionality and greatly enhanced speed. My original version was a bourne
# shell script.
#
# This script is released under GPL, the GNU General Public Licence. It is
# free, must remain free but has NO WARRANTY, neither expressed nor implied.
# It might work for you. The author(s) accepts no responsibility if it behaves
# wrongly, if it causes you damages or if you experience problems. Do report
# bugs, we might fix them.
#
# Home of the script development:
#
# http://spam.sourceforge.net
#
# Use this script from within i.e elm or pine. Select 'show all headers' and
# then pipe the offending mail into this script.
#

# You may need to change this if you're using anything else but Solaris:
$mailcmd="/usr/lib/sendmail";

# Linux example:
#$mailcmd="/usr/sbin/sendmail";

# This function checks for 0.13-style or earlier configuration files, and
# moves them to the new-style config in the ~/.spam subdirectory!
sub upgradefiles {
    if ( ((-r "$home/.spamcomplaint") ||
          (-r "$home/.spamfrom") ||
          (-r "$home/.spamfriends")) &&
         !(-d "$home/.spam") ) {
        print "NOTICE: Found old-style config files, upgrading them!\n";
        print "NOTICE: They are now found in $home/.spam\n";

        mkdir("$home/.spam", 0755); # create subdirectory
        $dir="$home/.spam";
 
        system("mv $home/.spamcomplaint $dir/complaint");
        system("mv $home/.spamfrom $dir/from");
        system("mv $home/.spamfriends $dir/friends");
    }
}

sub nslookup {
    my $host = $_[0];
    my $already = $cache{$host};
    my $newhost;
    if($already =~ /name:(.*)/) {
        # print "... resolve cache hit: $host\n";
        $newhost = $1;
    }
    else {
        my @digits = split (/\./, $host);
        my $address = pack ("C4", @digits);
        # print " ... resolving $host\n";
        $newhost = gethostbyaddr ($address, 2);
        $cache{$host}="name:$newhost";
    }
    return $newhost;
}

sub extractbodyhosts {
  for(@mailbody) {

    # while through a replace thing to match several links properly
    $line = $_;
    while(($line =~ s/http:\/\/([a-z0-9.]*)//i) ||
          ($line =~ s/www.([a-z0-9.]*)//i)) {
        $host = $1;
        if($host =~ /^((\d*)\.(\d*)\.(\d*)\.(\d*))$/) {
            # print "IPv4\n";
            $host = $1;
        }
        elsif(($host =~ /^(\d+)$/) && ($1>255)) {
            # print "weirdo compact IP\n";

            $rip = "";
            $ip = $1;
            for( 0 .. 3) {
                $s = 3 - $_;
                if($rip) {
                    $rip .= ".";
                }
                #printf "IP $ip & %08x\n", 0xff<<($s*8);
                $rip .= ($ip&(255<<($s*8)))>>($s*8);
            }
            
            $host = $rip;

        }
        else {
             # print "regular host\n";
        }

        if($host =~  /^((\d*)\.(\d*)\.(\d*)\.(\d*))$/) {

            $newhost = &nslookup($host);

            if($newhost) {
                $host = $newhost;
            }
        }

        # cut off 'www' to get a more likely mailable domain
        $host =~ s/^www.(.*)/$1/;
        if($host ne "") {
            if($debug) {
                print "BODY-HOST: $host\n";
            }
            AddHost($host);
        }
    }
}

}

sub mailaway {
    my $mailto = $_[0];
    my $envelopesender;
  
    if($fromline ne "") {
        if($fromline =~ /^.*?([^\s\@\<]+\@[^\s\>]+).*/) {
            $envelopesender = $1;
        }
        else {
            $envelopesender = "";
            print "WARNING: Couldn't get e-mail address from fromline: $fromline\n";
            print "WARNING: Omitting envelope sender from sendmail command line.\n";
        }
     }
    if($envelopesender ne "") {
        open(SENDIT, "|$mailcmd -t -f $envelopesender") ||
            die "Couldn't run mail program";
    }
    else {
        open(SENDIT, "|$mailcmd -t") ||
            die "Couldn't run mail program";
    }

    if(($insubject =~ /\=\?iso/i) || ($insubject eq "")) {
        # MIME code in subject, just leave it out of the complaint subject
        print SENDIT "Subject: Spam Report!\n";
    }
    else {
        print SENDIT "Subject: Spam Report: $insubject\n";        
    }

    if($fromline ne "") {
        print SENDIT "From: $fromline\n";
    }
    if($singlereceiver eq "") {
        # this is the actual list of receivers
        print SENDIT "To: $mailto\n";
    }
    else {
        print SENDIT "To: $mailto\n";
        print SENDIT "X-Couldbe-To: $receiver\n";
    }
    if($bccme) {
        print SENDIT "Bcc: $fromline\n";
    }
    print SENDIT "X-mailer: spam.pl $version - http://spam.sourceforge.net\n";
    print SENDIT "Precedence: bulk\nErrors-To: root\@localhost\n";
    print SENDIT "\n"; # header/body separator

    print SENDIT @body;

    if($bequiet) {
        print SENDIT "\n--- start of spam headers ---\n";
    }
    else {
        print SENDIT "\n--- start of spam ---\n";
    }
  
    for(@mailheader) {
        print SENDIT "$_\n";
    }

    if($bequiet) {
        print SENDIT "\n--- end of spam headers ---\n";
    }
    else {
        print SENDIT "\n";
        for(@mailbody) {
            print SENDIT "$_";
        }
        print SENDIT "\n--- end of spam ---\n";
    }

    if($nosig) {
    # user said no signature on command line.
    } else {
        $sig="$home/.signature";
        if(open(SIG, "<$sig")) {
            print SENDIT "-- \n";
            while(<SIG>) {
                print SENDIT $_;
            }
            close(SIG);
        }
    }
    
    close(SENDIT);
}

# we need $HOME at several places
$home=$ENV{'HOME'};
$spamdir="$home/.spam";

# parse the config file as plain perl:
do "$spamdir/config";

argv:

if($ARGV[0] eq "-h") {
    print <<EOF;
spam.pl version $version ($date)
Usage: $0 [-a][-b][-d][-h][-i][-m][-q][-w][single mail receiver]

 This program will try to extract information about which hosts that relayed
the abusive mail, and will sent off a complaint to the administrators of those
hosts. The mail should be sent to this program on stdin.

 -a makes the script use the report facility of abuse.net. This requires
    your From: address to be registered at that service prior to use.
    http://www.abuse.net/register.html

 -b Body. Extracts host names from the body part of the mail and includes
    those to the list of receivers.

 -d runs in debug-mode (shows a lot of internal processing choices).
    debug-mode does not send any mail.

 -e Edit list. Your \$EDITOR gets invoked with the list of found hosts before
    the mail is sent, to allow you to edit the list manually first.

 -h displays this help

 -i Included in the body of the message is the actual spam. This should be used
    where there is one e-mail abuse reporting address (e.g.,
    <reportabuse\@yourdomain.com>). Any '> ' sequences are removed.
 
 -m Metoo. Send a bcc: to myself as well as the normal receivers.
 
 -n No Signature. Do not attach \$HOME/.signature to outoing complaint message.

 -q Quiet. Only include the headers of the spam, rather than the entire body.

 -w Whois. Use the whois.abuse.net service to extract abuse emails from the
    offending domains.

 If you supply an argument to this program, that will be used as the single
receiver of the complaint mail, which can be used to test the correctness of
this script before using for real.

 NOTE that you are not included among the receivers of the complaint unless the
-m flag is specified.

 \$HOME/.spam/complaint will be used as mailbody in the complaint mail if
 existant

 \$HOME/.spam/friends should be a list with friendly domains that shouldn\'t
 receive complaints. Most probably your own site and other related ones.

 \$HOME/.spam/from should contain the From: email address of the complainer.

 \$HOME/.spam/addmailaddress should contain a list of all additional email
 address\'s to send the complaint to. In the US I suggest uce\@ftc.gov.

 \$HOME/.signature will be included at the bottom of the generated mail.
 This can be disabled with the -n flag.

     Short lesson in sendmail Received: headers
     ==========================================

Received: from host1 (host2 [ww.xx.yy.zz]) by host3 (8.7.5/8.7.3) with SMTP id
 MAA04298; Thu, 18 Jul 1996 12:18:06 -0600

host3 - The host that added the Received line
ww.xx.yy.zz - The IP address of the incoming SMTP connection
host2 - The reverse-DNS lookup of that IP address
host1 - The name the sender used in the SMTP HELO command when they connected

Author:  Daniel Stenberg <Daniel.Stenberg\@haxx.nu>
EOF
    exit;
}
elsif($ARGV[0] eq "-b") {
    $bodyextract^=1;
}
elsif($ARGV[0] eq "-w") {
    $whois^=1;
}
elsif($ARGV[0] eq "-e") {
    $editlist^=1;
}
elsif($ARGV[0] eq "-d") {
    $debug=1;
    print "DEBUG MODE ENABLED\n";
}
elsif($ARGV[0] eq "-dd") {
    $debug=1;
    $debug2=1; # extra detailed debug info
    print "DEBUG MODE ENABLED\n";
}
elsif($ARGV[0] eq "-a") {
    $abusenet ^= 1;
}
elsif($ARGV[0] eq "-m") {
    $bccme ^= 1;
}
elsif($ARGV[0] eq "-i") {
    $included ^= 1;
}
elsif($ARGV[0] eq "-q") {
    $bequiet ^= 1;
}
elsif($ARGV[0] eq "-n") {
    $nosig ^= 1;
}
elsif($ARGV[0] eq "-v") {
    print "spam.pl version $version ($date)\n";
    exit;
}

# This check must be after all switch checks:
elsif($ARGV[0] ne "") {
    $singlereceiver=$ARGV[0];
}

if($ARGV[0] ne "") {
    shift @ARGV;
    goto argv;
}

# change by eggert.ehmke@berlin.de
if ($debug2) {
    print "VERBOSE DEBUG MODE ENABLED\n";
}
elsif ($debug) {
    print "DEBUG MODE ENABLED\n";
}
if ($singlereceiver ne "") {
    print "Single receiver : $singlereceiver\n";
}

$inheader=0;

@mail=<STDIN>;

# check to see if we need to upgrade old config files
&upgradefiles();

# We support a file with domains that are KNOWN to never spam.
# This list should also include your own domain and friendly domains
# through which your mail may have been forwarded through with your
# approval and permission
#
# If my home domain is frontec.se and I forward my mail to ericsson.se
# its pretty important that both of those domains are added to prevent
# this script to send complaints to them too!
#
$friends="$spamdir/friends";

if(open(FRIENDS, "<$friends")) {
    @friends=<FRIENDS>;
    close(FRIENDS);
}
else {
    print "WARNING: You have no .spam/friends defined!\n";
    print "WARNING: This may generate complaints to friendly sites!\n";

    open(FRIENDS, ">$friends");
    print FRIENDS "";
    close(FRIENDS);
    print "WARNING: Exiting. Reinvoke if this is really what you want!\n";
    exit;
}

if($friends[0] eq "") {
    print "WARNING: You have no friendly domains entered in .spam/friends!\n";
    print "WARNING: This may generate complaints to friendly sites!\n";
}

if($debug) {
    for(@friends) {
	print "FRIEND: $_";
    }
}

#
# This is a small list with known two-name domains that should not get
# added as a mail receiving domain.
#
%knowntopdoms=('co.jp' => 1,
               'co.uk' => 1,
               'co.za' => 1,
               'com.mx' => 1,
               'com.sg' => 1);

sub AddHost {
    my $host = lc($_[0]);

    # sometimes we get trailing junk on the host name, like semicolons
    $host =~ s/([;:])+ *$//g;

    $host =~ s/\r//g; # there shouldn't be any such beasts here

    if($host =~ /\@/) {
        # @ in the host name looks like a fake
        return;
        # if this "host" has an @ letter, we cut it and everything to the
        # left of it
        # $host =~ s/([^@]*)@(.*)/$2/g;
    }
    
    # Hosts that have a digit in the toplevel domain!
    # Deal with IP address or bail out
    if($host =~ /\.[^.]*[0-9][^.]*$/) {
        $ip = $host;
        $host = &nslookup($host);
        if($host eq "") {
            if($debug2) {
                print "ERR: can't resolve IP $ip\n";
            }
            return;
        }
        if($debug2) {
            print "IP LOOKUP: $ip => $host\n";
        }
    }

    if($knowntopdoms{$host}) {
        if($debug2) {
            print "$host is known TLD, bailout!\n";
        }
        return;
    }

    for(@friends) {
	$friendly=$_;
	chomp $friendly;

	if($friendly =~ /\./) {
	    # A domain requires a dot. It allows for empty lines in the
	    # friends file
	    if($host =~ /$friendly$/i) {
		# known friendly domains are not added
                if($debug2) {
                    print "$host is friend, bailout!\n";
                }
		return;
	    }
	}
    }

    # since it is quite impossible to get the full domain name
    # from a local name, we just skip names that have no dots
    if($host =~ /\./) {		    
	if(!$offender{$host}) {
	    $offender{$host}=1;
	    push @uniqlist, $host;
            print "HOST: $host\n";
	}
    }
}

if ($included) {
    $blankline = $blankline2 = 0;
}

for(@mail) {
    if($included) {
        s/^\s*>\s*//;

        if($blankline2) {
            next;
        }

        if(!$blankline && /^\s*$/) {
            $blankline = 1;
            next;
        }
        elsif(!$blankline) {
            next;
        }

        if(!$inheader && /^[a-z0-9-_]*:/i) {
            $inheader = 1;
        }
        elsif(!$inheader) {
            next;
        }
    }
    if(!$inheader && ($_ =~ /^(|>)From /)) {
	$inheader=1;
    }
    elsif(($inheader<2) && ($_ =~ /^([A-Za-z_0-9-]*):/)) {
        # we may have found a header _without_ a previous "From " line.
        # this may happen when we pipe in a mail from a mail program since
        # those tend to strip the first from line
        # doesn't really matter, though...
	$inheader=1; 

        # save this line as a header
	$line = $_;
	chomp $line;
	push @concat, $line;
        push @mailheader, $line;
    }
    elsif(($inheader==1) && ($_ =~ /^[ \t]/)) {
	$line = $_;
	chomp $line;

	# replacing a sequence of beginning whitespaces with a single space
	$line =~ s/^[ \t]*/ /g;

	# append to previous line:
	$concat[$#concat]=$concat[$#concat].$line;
        push @mailheader, $line;
    }
    else {
        if($included) {
            if(!$blankline2 && /^\s*$/) {
                $blankline2 = 1;
                $inheader = 2;
            }
        }
        elsif($inheader == 1) {
	    $inheader=2;
	}
    }
    if($inheader == 2) {
        push @mailbody, $_;
    }
}

$rheaders=0;

#
# Extract hosts from the headers.
#
for(@concat) {
    if($_ =~ /^Received: /) {
	# This is a received line, deal with it
	#
	# EXAMPLE:
	# Received: from gade.imada.ou.dk (root@gade.imada.ou.dk
	# [130.225.128.158]) by gatekeeper.frontec.se (8.8.2/8.8.2) with ESMTP
	# id OAA07503 for    <Daniel.Stenberg@sth.frontec.se>;
	# Sun, 16 Aug 1998 14:28:03 +0200 (MET DST)

	$rheaders++;

	$rest = substr($_, 10);

	if($debug2) {
	    print "RECV: $rest\n";
	}
	if($rest =~ /^from ([^ ]*)/ ) {
	    # Received: from gade.imada.ou.dk
	    # add this host to the list of offenders

	    $host = $1;

	    # We have seen a from-host that looks like:
	    # "pobox.com(c5800-1-188.225ohio.megsinet.net[216.214.20.188])"
	    # we need to adjust to this weird thing. I.e no space between
	    # the host and the next info part

	    if( $host =~ /^([^(]*)/ ) {
		# get the "pobox.com" piece, get everything up to the first
		# parenthesis.
		$host = $1;
		$nextindex = 5;
	    }
	    else {
		$nextindex = 6;
	    }

	    AddHost($host);

	    $rest=substr($rest, length($host)+ $nextindex);

	    if($debug2) {
	      print "MORE: $rest\n";
	    }

	    if(($rest =~ /^ *\(([^\) ]*)/) ||
               ($rest =~ /[ -]*(\d+\.\d+\.\d+\.\d+)/)) {
		# (root@gade.imada.ou.dk [130.225.128.158])

		# we may get a "local" host name here, without full domain
		# we may also have a [ipnumber] to the right of the name
		# we may get an email address here like name@machine.domain
		# we may get an IP number only, like (192.192.192.192)
		# we may get weird stuff like this:
		#   "(root@[207.159.154.2])"
		# we may get the host and the IP-part merged like:
		#   "(c5800-1-188.225ohio.megsinet.net[216.214.20.188])"

                if($1 =~ /^(HELO |helo=)([^\) ]*)(.*)/) {
                    # qmail seems to add strings like this, and then the
                    # next string is the host instead!

                    #$rest =~ /^ *\(HELO ([^\) ]*)/;
                    $rest = $3;

                    if($debug2) {
                        print "qmail HELO host!\n";
                    }
                    $host = $2;
                    #$len = length($1);
                    #$rest = substr($rest, $len+8);
                }
                else {
                    $host = $1;
                    $len = length($1);
                    $rest = substr($rest, $len+3);

                }

		if($debug2) {
		    print "ENTRY: $host REST: $rest\n";
		}
                $prehost=$host;
		# check for IP-only:
		if($host =~ /^(\d+\.\d+\.\d+\.\d+)/) {
		    # HOST is nothing but IP!
                    $ip = $1;
		    if($debug2) {
			print "$ip is only IP address\n";
		    }
		    $host="";
		}

		# extract the possible IP number first:
		elsif($host=~ s/ *\[(\d+\.\d+\.\d+\.\d+)\]//) {
		    $ip = $1;
		    if($debug2) {
			print "HOST $prehost => IP $ip\n";
		    }
		}
		else {
		    $ip = "";
		}
		# If we have DNS access, we could look it up and if it resolves
		# we could add that as a host too.
                if($ip) {
                    AddHost($ip);
                }

		# remove the possible name@ part of the host:
		$host=~ s/([^@]*)@//g;
		
		if($host ne "") {
		    # we've seen occurrences when $host now contains
		    # "[207.159.154.2]" !!

		    # we check if anything behind the last dot is numeric,
		    # then we don't add the host since that means IP number
		    # and most likely the effect of a creation that looked
		    # like: "root@[207.159.154.2]" style.
		    if($host =~ /\..*[0-9]+$/) {
			if($debug2) {
			    print "BEEP: $host not a host name\n";
			}
		    }
		    else {
			AddHost($host);
		    }
		}

		if($debug2) {
		    print "REST: $rest\n";
		}

                # by gatekeeper.frontec.se (8.8.2/8.8.2)
                #  by pm1.contactor.se
                # [209.125.206.198]) by pm1.contactor.se (8.9.3/8.9.1)
		if($rest =~ /^.*by ([^ ]*)/ ) {
			$host = $1;
			if( $host =~ /^([^(]*)/ ) {
				# get the "pobox.com" piece, get everything up to the first
				# parenthesis.
				$host = $1;
			}
			if($debug) {
				print "HOST-B: $host\n";
			}
			AddHost($host);
		}
                # try the IP-only part:
		if($rest =~ /^\[(\d+\.\d+\.\d+\.\d+)\]/) {
                    if($debug2) {
                        print "IP: $1\n";
                    }
		    AddHost($1);
		}

	    }

	}
	elsif($rest =~ /^\(from ([^\)]*)/ ) {
	    # Received: (from dast@localhost)
#	    print "---- $1\n";
	}

    }
    elsif($_ =~ /^Subject: (.*)/i) {
        $insubject=$1;
    }
    else {
	#print $_."\n";
    }
}

if($rheaders == 0) {
    print "No Received: headers were found!\n";
    exit;
}

if($bodyextract) {
#
# Extract evil hosts from the spam mail body
#
    &extractbodyhosts();
}
if($uniqlist[0] eq "") {
    print "No unfriendly hosts were found. This cannot have been a spam!\n";
    exit;
}

# This creates a sequence of hosts from one.
#   penguin.wise.edt.ericsson.se
# will generate:
#
# penguin.wise.edt.ericsson.se
# wise.edt.ericsson.se
# edt.ericsson.se
# ericsson.se

sub generateaddress {
    $host = $_[0];

    @parts= split('\.', $host);

    while($#parts >= 1) {
	$new = join('.', @parts);
#	print "$new\n";

        AddHost($new);
	shift @parts;
    }
}

sub complainers {
    $host = $_[0];

    for('abuse', 'postmaster') {
	push @emails, $_."@".$host;
    }
}

sub abusecomplainers {
    $host = $_[0];

    # try to avoid building i.e first host.com@abuse.net
    # and then machine.host.com@abuse.net.
    # this hack attempts to skip the second action

    @p = split('\.', $host);

    $prts = $#p;

#    print "$host is $prts parts\n";

    $two=$p[$prts-1]."\.".$p[$prts];

    for(@emails) {
#	print "TRY $_ against $two\n";

	if($_ =~ /$two/) {
	    # 2nd already present
	    return;
	}
    }

    push @emails, $host."\@abuse.net";
}

if($editlist) {
    # we sort the list for human comfort
    @slist = sort { reverse($a) cmp reverse($b) } @uniqlist;
    my $file = "$home/.spam.$$";
    open(TMP, ">$file");
    
    for(@slist) {
        print TMP "$_\n";
    }
    close(TMP);

    $editor=$ENV{'EDITOR'};

    print "INVOKES \"$editor $file\"\n";
  
    $succ=system($editor, $file);

    print "SYSTEM: $succ\n";

    if(open(TMP, "<$file")) {
        undef @uniqlist;
        while(<TMP>) {
            chomp;
            push @uniqlist, $_;
        }
        close(TMP);
        system("rm $file");
        if($uniqlist[0] eq "") {
            print "BOO: all hosts removed, exiting!\n";
            exit;
        }
    }
}

# create all new combinations of hosts:
if(! $abusenet) {
    if ( $whois ) {
        %complaints = ();
        for(@uniqlist) {
            $badhost = $_;
            @list = `whois $badhost\@whois.abuse.net`;
            for ( @list ) {
                next if ( /^\[/ );
                if ( /([^\s]+)/ ) {
                     $complaints{$1} = 1;
                     if ( $debug ) {
                         print "WHOIS: $badhost => $1\n";
                     }
                 }
             }
         }
     }
     else {
         # if this is an abuse.net report, we don't do this
         for(@uniqlist) {
             #	if($debug) {
             #	    print "HOST: $_\n";
             #	}
             generateaddress($_);
         }
     }
}

# we sort the list 
@slist = sort { reverse($a) cmp reverse($b) } @uniqlist;

if(!$abusenet) {
    # make email addresses out of the hosts:
    if ( $whois ) {
        for ( keys %complaints ) {
            push @emails, $_;
        }
    }
    else {
        for(@slist) {
            complainers($_);
        }
    }
}
else {
    # the absue.net style of generating reports:
    for(@slist) {
	abusecomplainers($_);
    }
}

$addmailaddress="$spamdir/addmailaddress";

if(open(ADDMAIL, "<$addmailaddress")) {
	while (<ADDMAIL>) {
		chomp;
		push @emails, $_;
	}
}

# @emails is how a complete list of email addresses to complain to
#
# Now, get the predefined complaint mail or use our built-in.

$body="$spamdir/complaint";

if(open(BODY, "<$body")) {
    @body=<BODY>;
    close(BODY);
}
else {
    if($bequiet) {
        @body=('
Headers of an apparent spam attack using or referencing your network follow:
');
    }
    else {
    @body=('I recently received the following message, which appears to be from one of
your users or was relayed by one of your machines.  It looks to me like a
spam, unsolicited commercial e-mail.  Such mail is very annoying and widely
considered to be abusive.

If your domain is the source for this E-mail, could you encourage him/her/it
to cut it out?  Thanks.

If you are one of the relays down the chain, maybe you should consider to
restrict the use of your mail-server? Thanks.

If your host was faked or in any way spoofed, I trust you would still like to
know about it. Please regard this mail as information rather than
accusation. Forged headers are unfortunately sometimes used and you may have
received this mail even though you\'re completely innocent.

If you know who to tell or inform about this in order to make it stop. Please
forward this information to whom it may concern.

(This mail was automatically generated.)
');
    }
}


#
# Produce the complaint email
# 

$receiver = "";
for(@emails) {
  if($receiver ne "") {
    $receiver="$receiver, $_";
  }
  else {
    $receiver=$_;
  }
}

# If a single argument was specified, we mail the stuff to that address only
if($singlereceiver ne "") {
  $mailto=$singlereceiver;
}
else {
  $mailto=$receiver;
}

if($debug) {
  print "Spam-subject: $insubject\n";
  print "When run for real, this ";
}
print "sends mail to $mailto\n";

  if(open(EMAIL, "<$spamdir/from")) {
    # we found a From: line file.
    $fromline=<EMAIL>;
    chomp $fromline;
    close(EMAIL);
  }
  else {
    print "WARNING: You have no .spam/from defined!\n";
    print "WARNING: Your complaint mails don't get proper From: lines!\n";
    $fromline=""; # no from line
  }

if(!$debug) {
    if($abusenet) {
      for(@emails) {
        # a single mail for each offender!
        $receiver = $_;
	if($singlereceiver ne "") {
	  # for debugging, you can get it to mail a single specified addy
	  $mailto=$singlereceiver;
	}
	else {
	  $mailto=$receiver;
	}
        &mailaway($mailto);
      }
    }
    else {
      &mailaway($mailto);
    }
}
